#!/usr/bin/env python
#
# Copyright 2019 Scott Wales
#
# Author: Scott Wales <scott.wales@unimelb.edu.au>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import xarray
from pathlib import Path
from dask.diagnostics import ProgressBar
import dask
import numpy

def clean_dataset(infiles):
    #print(infiles)
    ds = xarray.open_mfdataset(infiles, combine='nested', concat_dim='time', chunks={'time': 30})
    bound_vars = []

    for name, var in ds.coords.items():
        bound_vars.append(var.attrs.pop('bounds',None))
        var.encoding['_FillValue'] = False

    for v in bound_vars:
        if v is not None:
            del ds[v]

    for name, var in ds.items():
        if '_FillValue' in var.encoding:
            var.encoding['missing_value'] = var.encoding['_FillValue']
        var.attrs.pop('grid_mapping', None)

        if var.attrs['units'] == 'mm':
            var.attrs['units'] = 'kg m-2 d-1'

        #ds[name] = var.where(ds[name] < numpy.inf).fillna(0)

    if 'crs' in ds:
        del ds['crs']

    #print(ds)
    return ds

def prepare_files(files):
    tmp = Path(os.environ['TMPDIR'])
    #tmp = Path('/g/data/w40/saw562/climpact/inputs')

    tasmin = tmp / 'tasmin.nc'
    tasmax = tmp / 'tasmax.nc'
    pr = tmp / 'pr.nc'

    tasks = []

    if len(files['pr']) > 0:
        pr_ds = clean_dataset(sorted(files['pr']))
        tasks.append(pr_ds.to_netcdf(pr, compute=False))
    else:
        pr = ''

    if len(files['tasmin']) > 0:
        tasmin_ds = clean_dataset(sorted(files['tasmin']))
        tasmax_ds = clean_dataset(sorted(files['tasmax']))
        tasks.append(tasmin_ds.to_netcdf(tasmin, compute=False))
        tasks.append(tasmax_ds.to_netcdf(tasmax, compute=False))

        xarray.testing.assert_equal(tasmin_ds.time, tasmax_ds.time)

        if len(files['pr']) > 0:
            xarray.testing.assert_equal(tasmin_ds.time, pr_ds.time)
    else:
        tasmin = ''
        tasmax = ''

    with ProgressBar():
        dask.compute(tasks)

    tasmin = ''
    tasmax = ''

    return tasmin, tasmax, pr
