% Jules Kajtar, UTas, jules.kajtar@utas.edu.au
% JBK 25/09/19
% function to detect MHWs: follows format of detect_mhw.m from m_mhw toolbox
% clim_s and clim_e: start and end dates of climatological base-line

function [MHW,mclim,m90,mhw_ts]=detect_mhw(temp,time,clim_s,clim_e,time_cal)

if strcmp(time_cal,'360')
 dy=360;
elseif strcmp(time_cal,'noleap')
 dy=365;
elseif strcmp(time_cal,'standard')
 dy=365;
end

% for building climatological seasonal cycle and threshold,
% trim to required start and end dates
[i1,i2]=findrange(time,clim_s,clim_e);
sst_tmp=temp(:,:,i1:i2);
time_tmp=time(i1:i2);

% with standard calendar, for climatology, temporarily ignore Feb 29
if strcmp(time_cal,'standard')
 ii=find(month(time_tmp)==2 & day(time_tmp)==29);
 sst_tmp(:,:,ii)=[];
end

% compute seasonal cycle at each grid-point
% for each calendar day, average over an 11-day period around that date
S=size(sst_tmp);
% this assumes that climatological period starts on Jan 1, and contains
% an integer number of complete calendar years
idoy=repmat([1:dy]',[S(3)/dy 1]);
hw=5;   % same as vWindowHalfWidth in m_mhw
clear scyc thres;
for kk=1:dy
 % range of indices to average over
 inds=kk-hw:kk+hw;
 
 % for indices near start/end of calendar, wrap
 inds(inds<1)=inds(inds<1)+dy;
 inds(inds>dy)=inds(inds>dy)-dy;

 sst1=sst_tmp(:,:,ismember(idoy,inds));
 scyc(:,:,kk)=mean(sst1,3);
 thres(:,:,kk)=quantile(sst1,0.9,3);
end

% smooth the resulting seasonal cycle
smth_len=31;   % same as vsmoothPercentileWidth in m_mhw
scyc_pad=repmat(scyc,[1 1 3]); % repeat seasonal cycle 3 times for smoothing
scyc_pad=movmean(scyc_pad,smth_len,3); % smoothdata introduced in R2017a
scyc=scyc_pad(:,:,(dy+1):(dy+1)+(dy-1));  % retain only central cycle

% smooth the threshold
thres_pad=repmat(thres,[1 1 3]); % repeat seasonal cycle 3 times for smoothing
thres_pad=movmean(thres_pad,smth_len,3); % smoothdata introduced in R2017a
thres=thres_pad(:,:,(dy+1):(dy+1)+(dy-1));  % retain only central cycle
clear scyc_pad thres_pad;

% for standard calendar
% create temporary seasonal cycles and thresholds for leap year
% fill in Feb 29 with average of Feb 28 and Mar 1
if strcmp(time_cal,'standard')
 scyc_ly=cat(3,scyc(:,:,1:59),mean(scyc(:,:,[59 60]),3),scyc(:,:,60:end));
 thres_ly=cat(3,thres(:,:,1:59),mean(thres(:,:,[59 60]),3),thres(:,:,60:end));
end

% now map the seasonal cycle and threshold to the original calendar
kk=find(month(time)==1 & day(time)==1);  % find all Jan 1
clear scyc_ts thres_ts;

% for standard calendar, need to treat leap years separately
if strcmp(time_cal,'standard')
 ily=leapyear(year(time(kk)));  % check which years are leap years
 for ii=1:numel(kk)
  if ily(ii)==1   % map leap years
   scyc_ts(:,:,kk(ii):kk(ii)+dy)=scyc_ly;
   thres_ts(:,:,kk(ii):kk(ii)+dy)=thres_ly;
  else            % map non leap years
   scyc_ts(:,:,kk(ii):kk(ii)+(dy-1))=scyc;
   thres_ts(:,:,kk(ii):kk(ii)+(dy-1))=thres;
  end
 end
% for other calendars, just map based on year length
else
 for ii=1:numel(kk)
  scyc_ts(:,:,kk(ii):kk(ii)+(dy-1))=scyc;
  thres_ts(:,:,kk(ii):kk(ii)+(dy-1))=thres;
 end
end

% subtract the seasonal cycle from the input data
sst=temp-scyc_ts;

% find where SST exceeds the threshold
ii=single(sst>thres_ts-scyc_ts);
ii(ii==0)=nan;
sst_90=sst.*ii;

% searching for marine heatwaves
% find datapoints across whole grid that exceed 90%
% loop over each point, find points with greatest number of extreme neighbours
% then could find points with 

% clear arrays;
mhw_cum=0;
mhw_max=0;
mhw_dur=0;
mhw_start=NaT;
mhw_end=NaT;
mhw_ts=nan(size(sst_90));
S=size(sst);

for jj=1:S(2)
 fl=tic;
 disp(sprintf('%d/%d',jj,S(2)));
 for ii=1:S(1)
  
  % initialise counter
  n_ev=0;
  
  % store index of SST above 90% threshold
  idx=squeeze(sst(ii,jj,:));
  idx_90=squeeze(sst_90(ii,jj,:));
  
  % only search where not all values are NaN
  if ~all(isnan(idx_90))
   % create check array, of 1s and 0s, used for searching for MHWs
   chk=idx_90(:);
   idx=idx(:);
   chk(~isnan(chk))=1;
   chk(isnan(chk))=0;
  
   % 1) detect start, end, and duration of all events
   chk_pad=[0; chk; 0];     % pad with zeroes to check for events at start/end
   chk_d=diff(chk_pad);   % detect switches between 0 <-> 1
   evt_s=find(chk_d==1);  % starts of target events
   evt_e=find(chk_d==-1); % ends of target events
   evt_d=evt_e-evt_s;     % event lengths

   % 2) find and remove events of less than 5 days
   ifnd=find(evt_d<5);
   evt_d(ifnd)=[]; evt_s(ifnd)=[]; evt_e(ifnd)=[];

   % 3) combine periods separated by two days or less
   ifnd=find(evt_s(2:end)-evt_e(1:end-1) < 3); % find events seperated by < 3 days
   evt_s(ifnd+1)=[]; evt_e(ifnd)=[];  % remove the start/end of such events
   evt_d=evt_e-evt_s;             % update durations of events
   
   % 4) compute cumulative and max intensity
   % use idx, instead of idx_90, because idx_90 contains nans for events broken
   % by less than 3 days
   n_ev=length(evt_d);  % number of events
   
   % subtract 1 from end date, so that date coincides with the last day of event
   % and not the day after
   % but calculation of event duration is fine
   evt_e=evt_e-1;
   
   clear evt_ic evt_im;
   for kk=1:n_ev
    evt_ic(kk)=sum(idx(evt_s(kk):evt_e(kk)));  % cumulative intensity
    evt_im(kk)=max(idx(evt_s(kk):evt_e(kk)));  % mean intensity
    mhw_ts(ii,jj,evt_s(kk):evt_e(kk))=idx(evt_s(kk):evt_e(kk));
   end
  end
   
  if n_ev>0
   mhw_cum(ii,jj,1:n_ev)=evt_ic;
   mhw_max(ii,jj,1:n_ev)=evt_im;
   mhw_dur(ii,jj,1:n_ev)=evt_d;
   mhw_start(ii,jj,1:n_ev)=time(evt_s);
   mhw_end(ii,jj,1:n_ev)=time(evt_e);
  else
   mhw_cum(ii,jj,1)=0;
   mhw_max(ii,jj,1)=0;
   mhw_dur(ii,jj,1)=0;
   mhw_start(ii,jj,1)=NaT;
   mhw_end(ii,jj,1)=NaT;
  end
 end
 toc(fl)
end

% reformat dates for table
mhw_start.Format='dd-MM-yy';
mhw_end.Format='dd-MM-yy';

% sort MHW events, and get indices
[C,I]=sort(mhw_cum(:),'descend');
[ilon,ilat,itpos]=ind2sub(size(mhw_cum),I);

imhw=I;  % ranked MHW indices

% remove zero values
ii=find(C==0);
imhw(ii)=[];
ilon(ii)=[];
ilat(ii)=[];

mhw_start=squeeze(mhw_start(imhw));
mhw_end=squeeze(mhw_end(imhw));
mhw_dur=squeeze(mhw_dur(imhw));
mhw_max=squeeze(mhw_max(imhw));
mhw_cum=squeeze(mhw_cum(imhw));

MHW=table(mhw_start,mhw_end,mhw_dur,mhw_max,mhw_cum,ilon,ilat,'VariableNames',{'onset','term','dur','int_max','int_cum','x','y'});

if strcmp(time_cal,'standard')
 mclim=scyc_ly;
 m90=thres_ly;
else
 mclim=scyc;
 m90=thres;
end

return

